package httpsdemo;

import java.util.*;
import java.io.*;
import java.nio.charset.Charset;
import java.security.KeyStore;
import java.security.cert.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.util.EntityUtils;
import java.text.SimpleDateFormat;
import org.slf4j.Logger;

public class HttpClientDemo {
    private static String proxyHost = null;
    private static int proxyPort = 0;
    private static String proxyUsername = null;
    private static String proxyPassword = null;
    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private static Logger logger = LoggerUtils.getAppLogger();
	
    private static KeyStore loadKeyStore(String filename, String password) throws Exception {
    	logger.info("load keystore {}", filename);
		KeyStore keyStore = KeyStore.getInstance(KeyStore.getDefaultType());
		File file = new File(filename);
		InputStream keyStoreInput = new FileInputStream(file);
		keyStore.load(keyStoreInput, password.toCharArray());
		keyStoreInput.close();
		return keyStore;
    }
    
    private static void printCert(Certificate cert) throws Exception {
    	if (cert instanceof X509Certificate) {
			 X509Certificate x509 = (X509Certificate)cert;
			 logger.info("issuer: {}", x509.getIssuerX500Principal());
			 logger.info("subject: {}", x509.getSubjectX500Principal());
			 logger.info("signed date: {}", sdf.format(x509.getNotBefore()));
			 logger.info("expired date: {}", sdf.format(x509.getNotAfter()));
			 logger.info("hash: {}", DigestUtils.sha1Hex(x509.getEncoded()));
		 } else {
			 logger.info("cert type: {}", cert.getType());
			 logger.info("cert: {}", cert.toString());
		 }
    }
    
    private static void printKeyStore(KeyStore keyStore) throws Exception {
    	 Enumeration<String> enums = keyStore.aliases();
    	 while (enums.hasMoreElements()) {
    		 String alias = enums.nextElement();
    		 if (keyStore.isCertificateEntry(alias)) {
    			 logger.info("CertificateEntry: {}", alias);
    			 Certificate cert = keyStore.getCertificate(alias);	 
    			 printCert(cert);
    		 }
    	 }
    	 enums = keyStore.aliases();
    	 while (enums.hasMoreElements()) {
    		 String alias = enums.nextElement();
    		 if (keyStore.isKeyEntry(alias)) {
    			 logger.info("KeyEntry: {}", alias);
    			 Certificate cert = keyStore.getCertificate(alias);	 
    			 printCert(cert);
    		 }
    	 }
    }
    
    public static void main(String[] args) throws Exception {
    	if (args.length < 2) {
    		System.out.println("Usage: command url serverJks[:password] [clientJks[:password]]");
    		System.exit(-1);
    	}
    	
    	String url = args[0];
    	String serverJks = args[1];
    	String clientJks = null;
    	if (args.length >= 3)
    		clientJks = args[2];
    	
    	String serverJksPass = "Abcd1234";
        String clientJksPass = "Abcd1234";
        if (serverJks!=null && serverJks.contains(":")) {
        	String[] parts = serverJks.split(":");
        	serverJks = parts[0];
        	serverJksPass = parts[1];
        }
        if (clientJks!=null && clientJks.contains(":")) {
        	String[] parts = clientJks.split(":");
        	clientJks = parts[0];
        	clientJksPass = parts[1];
        }
        
        logger.info("url is {}", url);
    	logger.info("server jks is {}", serverJks);
    	logger.info("client jks is {}", clientJks);
        
        KeyStore trustStore = null;
        KeyStore clientStore = null;
        trustStore = loadKeyStore(serverJks, serverJksPass);
        printKeyStore(trustStore);
        if (clientJks != null) {
        	clientStore = loadKeyStore(clientJks, clientJksPass);
        	printKeyStore(clientStore);
        }
        
        HttpClient client = HttpClientUtils.createHttpClientWithCert(trustStore, clientStore, clientJksPass,
        		200, 200, 1000, 3000, proxyHost, proxyPort, proxyUsername, proxyPassword);
        HttpGet httpGet = new HttpGet(url);
        HttpResponse response = client.execute(httpGet);
        System.out.println(response.getStatusLine());
        System.out.println(EntityUtils.toString(response.getEntity(), Charset.defaultCharset()));
    }
}

